@@ -49,6 +49,8 @@
 #include <linux/rtnetlink.h>
 #ifdef HAVE_LINUX_WIRELESS_H
 #include <linux/wireless.h>
+#else
+#include <net/if.h>
 #endif
 #include <sys/ioctl.h>
 #include <errno.h>
