@@ -171,7 +171,7 @@ subscription_data_free (SubscriptionData *data)
         }
        
         /* Further cleanup */
-        g_list_free_full (data->callbacks, g_free);
+        g_list_free_full (data->callbacks, (GDestroyNotify) soup_uri_free);
 
         g_free (data->sid);
 
@@ -1181,6 +1181,31 @@ send_initial_state (SubscriptionData *data)
         g_free (mem);
 }
 
+static GList *
+add_subscription_callback (GUPnPContext *context,
+                           GList *list,
+                           const char *callback)
+{
+            SoupURI *local_uri = NULL;
+            const char *host = NULL;
+
+            local_uri = soup_uri_new (callback);
+            if (local_uri == NULL) {
+                    return list;
+            }
+
+
+            host = soup_uri_get_host (local_uri);
+            // CVE-2020-12695: Ignore subscription call-backs that are not "in
+            // our network segment"
+            if (gupnp_context_ip_is_ours (context, host)) {
+                    return g_list_append (list, local_uri);
+            } else {
+                    g_warning ("%s is not in our network; ignoring", callback);
+            }
+
+            return list;
+}
 
 /* Subscription request */
 static void
@@ -1189,13 +1214,21 @@ subscribe (GUPnPService *service,
            const char   *callback)
 {
         SubscriptionData *data;
-        char *start, *end, *uri;
+        char *start, *end;
+        GUPnPContext *context;
+        int callbacks = 0;
+
+        context = gupnp_service_info_get_context
+                                            (GUPNP_SERVICE_INFO (service));
 
         data = g_slice_new0 (SubscriptionData);
 
         /* Parse callback list */
         start = (char *) callback;
-        while ((start = strchr (start, '<'))) {
+
+        // Arbitrarily limit the list of callbacks to 6
+        // Part of CVE-2020-12695 mitigation
+        while (callbacks < 6 && (start = strchr (start, '<'))) {
                 start += 1;
                 if (!start || !*start)
                         break;
@@ -1204,12 +1237,24 @@ subscribe (GUPnPService *service,
                 if (!end || !*end)
                         break;
 
-                if (strncmp (start, "http://", strlen ("http://")) == 0) {
-                        uri = g_strndup (start, end - start);
-                        data->callbacks = g_list_append (data->callbacks, uri);
+                *end = '\0';
+                if (g_str_has_prefix (start, "http://")) {
+                        // DLNA 7.3.2.24.4 - URIs shall not exceed 256 bytes
+                        // Also one part of CVE-2020-12695 mitigation - limit URI length
+                        // UPnP does not impose any restrictions here
+                        if (strlen (start) <= 256) {
+                                data->callbacks = add_subscription_callback (context,
+                                                                             data->callbacks,
+                                                                             start);
+                        } else {
+                                g_warning ("Subscription URI exceeds recommended length "
+                                           "of 256 bytes, skipping");
+                        }
                 }
+                *end = '>';
 
                 start = end;
+                callbacks++;
         }
 
         if (!data->callbacks) {
@@ -1914,32 +1959,27 @@ notify_got_response (G_GNUC_UNUSED SoupSession *session,
 }
 
 /* Send notification @user_data to subscriber @value */
-static void
-notify_subscriber (G_GNUC_UNUSED gpointer key,
-                   gpointer               value,
-                   gpointer               user_data)
-{
-        SubscriptionData *data;
-        const char *property_set;
-        char *tmp;
-        SoupMessage *msg;
-        SoupSession *session;
+    static void
+    notify_subscriber (G_GNUC_UNUSED gpointer key,
+                       gpointer               value,
+                       gpointer               user_data)
+    {
+            SubscriptionData *data;
+            const char *property_set;
+            char *tmp;
+            SoupMessage *msg;
+            SoupSession *session;
 
-        data = value;
-        property_set = user_data;
+            data = value;
+            property_set = user_data;
 
-        /* Subscriber called unsubscribe */
-        if (subscription_data_can_delete (data))
-                return;
+            /* Subscriber called unsubscribe */
+            if (subscription_data_can_delete (data))
+                    return;
 
-        /* Create message */
-        msg = soup_message_new (GENA_METHOD_NOTIFY, data->callbacks->data);
-        if (!msg) {
-                g_warning ("Invalid callback URL: %s",
-                           (char *) data->callbacks->data);
-
-                return;
-        }
+            /* Create message */
+            msg = soup_message_new_from_uri (GENA_METHOD_NOTIFY,
+                                             data->callbacks->data);
 
         soup_message_headers_append (msg->request_headers,
                                      "NT",
